"""
Core Linear Algebra Tools
=========================

=============== ==========================================================
Linear algebra basics
==========================================================================
norm            Vector or matrix norm
inv             Inverse of a square matrix
solve           Solve a linear system of equations
det             Determinant of a square matrix
slogdet         Logarithm of the determinant of a square matrix
lstsq           Solve linear least-squares problem
pinv            Pseudo-inverse (Moore-Penrose) calculated using a singular
                value decomposition
matrix_power    Integer power of a square matrix
matrix_rank     Calculate matrix rank using an SVD-based method
=============== ==========================================================

=============== ==========================================================
Eigenvalues and decompositions
==========================================================================
eig             Eigenvalues and vectors of a square matrix
eigh            Eigenvalues and eigenvectors of a Hermitian matrix
eigvals         Eigenvalues of a square matrix
eigvalsh        Eigenvalues of a Hermitian matrix
qr              QR decomposition of a matrix
svd             Singular value decomposition of a matrix
cholesky        Cholesky decomposition of a matrix
=============== ==========================================================

=============== ==========================================================
Tensor operations
==========================================================================
tensorsolve     Solve a linear tensor equation
tensorinv       Calculate an inverse of a tensor
=============== ==========================================================

=============== ==========================================================
Exceptions
==========================================================================
LinAlgError     Indicates a failed linear algebra operation
=============== ==========================================================

"""
from __future__ import division, absolute_import, print_function

# To get sub-modules
from .info import __doc__

from .linalg import *

from numpy.testing.nosetester import _numpy_tester
test = _numpy_tester().test
bench = _numpy_tester().bench



from __future__ import division, absolute_import, print_function

import sys

import numpy as np
from numpy.ctypeslib import ndpointer, load_library
from numpy.distutils.misc_util import get_shared_lib_extension
from numpy.testing import TestCase, run_module_suite, dec

try:
    cdll = None
    if hasattr(sys, 'gettotalrefcount'):
        try:
            cdll = load_library('multiarray_d', np.core.multiarray.__file__)
        except OSError:
            pass
    if cdll is None:
        cdll = load_library('multiarray', np.core.multiarray.__file__)
    _HAS_CTYPE = True
except ImportError:
    _HAS_CTYPE = False

class TestLoadLibrary(TestCase):
    @dec.skipif(not _HAS_CTYPE,
                "ctypes not available on this python installation")
    @dec.knownfailureif(sys.platform ==
                        'cygwin', "This test is known to fail on cygwin")
    def test_basic(self):
        try:
            # Should succeed
            load_library('multiarray', np.core.multiarray.__file__)
        except ImportError as e:
            msg = ("ctypes is not available on this python: skipping the test"
                   " (import error was: %s)" % str(e))
            print(msg)

    @dec.skipif(not _HAS_CTYPE,
                "ctypes not available on this python installation")
    @dec.knownfailureif(sys.platform ==
                        'cygwin', "This test is known to fail on cygwin")
    def test_basic2(self):
        # Regression for #801: load_library with a full library name
        # (including extension) does not work.
        try:
            try:
                so = get_shared_lib_extension(is_python_ext=True)
                # Should succeed
                load_library('multiarray%s' % so, np.core.multiarray.__file__)
            except ImportError:
                print("No distutils available, skipping test.")
        except ImportError as e:
            msg = ("ctypes is not available on this python: skipping the test"
                   " (import error was: %s)" % str(e))
            print(msg)

class TestNdpointer(TestCase):
    def test_dtype(self):
        dt = np.intc
        p = ndpointer(dtype=dt)
        self.assertTrue(p.from_param(np.array([1], dt)))
        dt = '<i4'
        p = ndpointer(dtype=dt)
        self.assertTrue(p.from_param(np.array([1], dt)))
        dt = np.dtype('>i4')
        p = ndpointer(dtype=dt)
        p.from_param(np.array([1], dt))
        self.assertRaises(TypeError, p.from_param,
                          np.array([1], dt.newbyteorder('swap')))
        dtnames = ['x', 'y']
        dtformats = [np.intc, np.float64]
        dtdescr = {'names': dtnames, 'formats': dtformats}
        dt = np.dtype(dtdescr)
        p = ndpointer(dtype=dt)
        self.assertTrue(p.from_param(np.zeros((10,), dt)))
        samedt = np.dtype(dtdescr)
        p = ndpointer(dtype=samedt)
        self.assertTrue(p.from_param(np.zeros((10,), dt)))
        dt2 = np.dtype(dtdescr, align=True)
        if dt.itemsize != dt2.itemsize:
            self.assertRaises(TypeError, p.from_param, np.zeros((10,), dt2))
        else:
            self.assertTrue(p.from_param(np.zeros((10,), dt2)))

    def test_ndim(self):
        p = ndpointer(ndim=0)
        self.assertTrue(p.from_param(np.array(1)))
        self.assertRaises(TypeError, p.from_param, np.array([1]))
        p = ndpointer(ndim=1)
        self.assertRaises(TypeError, p.from_param, np.array(1))
        self.assertTrue(p.from_param(np.array([1])))
        p = ndpointer(ndim=2)
        self.assertTrue(p.from_param(np.array([[1]])))

    def test_shape(self):
        p = ndpointer(shape=(1, 2))
        self.assertTrue(p.from_param(np.array([[1, 2]])))
        self.assertRaises(TypeError, p.from_param, np.array([[1], [2]]))
        p = ndpointer(shape=())
        self.assertTrue(p.from_param(np.array(1)))

    def test_flags(self):
        x = np.array([[1, 2], [3, 4]], order='F')
        p = ndpointer(flags='FORTRAN')
        self.assertTrue(p.from_param(x))
        p = ndpointer(flags='CONTIGUOUS')
        self.assertRaises(TypeError, p.from_param, x)
        p = ndpointer(flags=x.flags.num)
        self.assertTrue(p.from_param(x))
        self.assertRaises(TypeError, p.from_param, np.array([[1, 2], [3, 4]]))

    def test_cache(self):
        a1 = ndpointer(dtype=np.float64)
        a2 = ndpointer(dtype=np.float64)
        self.assertEqual(a1, a2)


if __name__ == "__main__":
    run_module_suite()



import random

import numpy as np
try:
    from numpy.fft import using_mklfft
except:
    using_mklfft = None
from numpy.testing import assert_array_almost_equal, TestCase, run_module_suite


def fft1(x):
    L = len(x)
    phase = -2j * np.pi * (np.arange(L) / float(L))
    phase = np.arange(L).reshape(-1, 1) * phase
    return np.sum(x * np.exp(phase), axis=1)


class TestFFT1D(TestCase):
    def test_basic(self):
        rand = np.random.random
        x = rand(30) + 1j * rand(30)
        assert_array_almost_equal(fft1(x), np.fft.fft(x))

    def test_change_array(self):
        rand = np.random.random
        x = rand(10) + 1j * rand(10)
        for n in range(5, 15):
            xx = x.copy()
            np.fft.fft(x, n)
            assert_array_almost_equal(x, xx)

    def test_narg(self):
        rand = np.random.random
        x = rand(10) + 1j * rand(10)
        for n in range(5, 15):
            if n < len(x): # crop
                y = x[:n]
            else: # pad with zeros
                y = list(x) + ((n - len(x)) * [0])
            self.assertEqual(n, len(y))
            z = np.fft.fft(x, n)
            self.assertEqual(n, len(z))
            assert_array_almost_equal(fft1(y), z)

    def test_dtype(self):
        for t1, t2 in [(np.float32, 'complex64'),
                       (np.float64, 'complex128'),
                       (np.complex64, 'complex64'),
                       (np.complex128, 'complex128')]:
            a = np.exp(2j * np.pi * np.arange(8) / 8)
            a = a.astype(t1)
            b = np.fft.fft(a)
            assert_array_almost_equal(fft1(a), b)
            #if using_mklfft:
            self.assertEqual(b.dtype.name, t2)
            #else:
                #self.assertEqual(b.dtype.name, 'complex128')

    def test_sizes(self):
        for n in [1, 2, 3, 4, 5, 8, 16, 20, 31, 32, 127, 128, 129]:
            x = np.random.random(n) + 1j * np.random.random(n)
            assert_array_almost_equal(fft1(x), np.fft.fft(x))

    def test_ifft(self):
        for n in range(2, 32):
            x = np.random.random(n) + 1j * np.random.random(n)
            y = np.fft.fft(x)
            z = np.fft.ifft(y)
            assert_array_almost_equal(x, z)

    def test_rfft(self):
        x = np.fft.rfft([0, 1, 0, 0])
        y = np.array([1, -1j, -1])
        assert_array_almost_equal(x, y)

        x = np.fft.rfft([0, 1, 0], n=4)
        assert_array_almost_equal(x, y)

    def test_irfft(self):
        for x, y in [([1, 0], [0.5, 0.5]),
                     ([1, -1j, -1], [0, 1, 0, 0]),
                     ([2, 0, 2, 0], [1, 0, 0, 1, 0, 0]),
                     ]:
            assert_array_almost_equal(np.fft.irfft(x), y)

        for m in range(2, 20):
            x = [1] + ([0] * (m - 1))
            n = 2 * (m - 1)
            y = [1.0 / n] * n
            assert_array_almost_equal(np.fft.irfft(x), y)
            assert_array_almost_equal(np.fft.irfft(x, n=n), y)

        for n in range(2, 32):
            x = np.random.random(n)
            y = np.fft.rfft(x)
            z = np.fft.irfft(y, n=n)
            assert_array_almost_equal(x, z)

    def test_2d_array(self):
        m, n = 5, 3
        a = np.random.randn(m, n) + 1j * np.random.randn(m, n)
        b = np.array([np.fft.fft(a[i]) for i in range(m)])
        assert_array_almost_equal(np.fft.fft(a), b)

        b = np.swapaxes(a, 0, 1)
        c = np.fft.fft(b)
        d = np.swapaxes(c, 0, 1)
        assert_array_almost_equal(np.fft.fft(a, axis=0), d)


class TestFFTND(TestCase):
    def test_basic_2d(self):
        a = np.random.randn(5, 3) + 1j * np.random.randn(5, 3)
        b = a.copy()
        for i in range(2):
            b = np.array(np.fft.fft(b, axis=i))
        assert_array_almost_equal(np.fft.fftn(a), b)

    def test_3d_1d(self):
        a = np.random.randn(5, 3, 7) + 1j * np.random.randn(5, 3, 7)
        for axis in range(3):
            b = np.array(np.fft.fft(a, axis=axis))
            assert_array_almost_equal(np.fft.fftn(a, axes=(axis,)), b)

    def test_3d_2d(self):
        a = np.random.randn(5, 3, 7) + 1j * np.random.randn(5, 3, 7)
        b = a.copy()
        axes = (1, 2)
        for i in axes:
            b = np.array(np.fft.fft(b, axis=i))
        assert_array_almost_equal(np.fft.fftn(a, axes=axes), b)

    def test_nd_dim(self):
        for nd in range(1, 9):
            args = tuple(nd * [3])
            a = np.random.randn(*args) + 1j * np.random.randn(*args)
            for dim in range(1, nd + 1):
                b = a.copy()
                axes = range(nd - dim, nd)
                for i in axes:
                    b = np.array(np.fft.fft(b, axis=i))
                assert_array_almost_equal(np.fft.fftn(a, axes=axes), b)

    def test_nd_axes(self):
        for nd in range(1, 9):
            args = tuple(nd * [3])
            a = np.random.randn(*args) + 1j * np.random.randn(*args)
            for dim in range(1, nd + 1):
                b = a.copy()
                axes = random.sample(range(nd), dim)
                for i in axes:
                    b = np.array(np.fft.fft(b, axis=i))
                assert_array_almost_equal(np.fft.fftn(a, axes=axes), b)

    def test_nd_shape(self):
        for nd in range(1, 9):
            args = tuple(nd * [3])
            a = np.random.randn(*args) + 1j * np.random.randn(*args)
            for dim in range(1, nd + 1):
                b = a.copy()
                axes = random.sample(range(nd), dim)
                shape = [random.randint(2, 5) for dum in range(dim)]
                for i in range(dim):
                    b = np.array(np.fft.fft(b, n=shape[i], axis=axes[i]))
                assert_array_almost_equal(np.fft.fftn(a, s=shape, axes=axes), b)

    def test_basic_nd(self):
        for nd in range(1, 9):
            args = tuple(nd * [3])
            a = np.random.randn(*args) + 1j * np.random.randn(*args)
            b = a.copy()
            for i in range(nd):
                b = np.array(np.fft.fft(b, axis=i))
            assert_array_almost_equal(np.fft.fftn(a), b)

    def test_ifftn(self):
        for nd in range(1, 9):
            args = tuple(nd * [3])
            x = np.random.randn(*args) + 1j * np.random.randn(*args)
            y = np.fft.fftn(x)
            z = np.fft.ifftn(y)
            assert_array_almost_equal(x, z)


if __name__ == "__main__":
    run_module_suite()
